%addpres - Function to add pressure to rough data file
function addpres(FileName,FileType);	%Function to scale rough data file
%dbstop in interp1AGA
%Inputs:
%FileName	- Filename with extension
%FileType	- Axon text file with 2 channels <1*>, Axon text file with 1 channel <2>, text file <3>
ReadStep=200000;	%Data portion to read from HDD in a step

FileNamePres=[FileName,'_prs.txt'];	%Pressure file name
FileNameOut=[FileName,'_addprs.'];	%Data output filename
fidDF=fopen(FileName);	%Opens trace file
fidPF=fopen(FileNamePres);	%Opens pressure file
fidPS=fopen('pressure sample.atf');	%Opens pressure sample file


%***	Creating sample pressure matrix
for i=1:10	%Reading text file header
   PresNotes{i}=fgets(fidPS);
end
PS=[0;0];	%Initializes sample pressure matrix
Read=2*ReadStep;
while Read>=2*ReadStep	%If the last time complete ReadStep length was read
   [PS,Read]=fscanf(fidPS,'%g %g',[2 ReadStep]);
end
PS=PS(2,:);	%Pressure column.
fclose(fidPS);	%Closes Pressure Sample file



%***	Creating Time-Pressure line matrix
for i=1:5	%Reading text file header
   PresNotes{i}=fgets(fidPF);
end
PresFileType=sscanf(PresNotes{2},'%g');	%Pressure file type (1* - change times, 2 - full record)
TransTime=sscanf(PresNotes{4},'%g');	%Transition time, s (for pressure file type 1)
PF=[0;0];	%Initializes Pressure File matrix
Read=2*ReadStep;
while Read>=2*ReadStep	%If the last time complete ReadStep length was read
   [PF,Read]=fscanf(fidPF,'%g %g',[2 ReadStep]);
end
fclose(fidPF);	%Closes Pressure file

switch PresFileType
case 1	%change times type. Generating time-pressure curve
   TimePres=[PF(1,1);PF(2,1)];	%Initializes Time-Pressure matrix
   for i=2:length(PF)
      TimePres(1,(2*i-2))=PF(1,i)-TransTime;	%Generating last point before current pressure transition
      TimePres(2,(2*i-2))=PF(2,i-1);
      TimePres(1,(2*i-1))=PF(1,i);	%Adding first point after current pressure transition
      TimePres(2,(2*i-1))=PF(2,i);
   end
case 2
   TimePres=PF;
end
TimePres(1,end+1)=TimePres(1,end)+1;	%Adding one more horizontal point for pressure extrapolation
TimePres(2,end)=TimePres(2,end-1);

max3=max(PS)+max(TimePres(2,:));	%Maximum possible pressure value
min3=min(PS)+min(TimePres(2,:));	%Minimum possible pressure value


%***	Adding pressure to the rough data file


switch FileType
case 1	%Axon Text File (.ATF) with 2 channels
   
case 2	%Axon Text File (.ATF) with 1 channel
   FileNameOut=[FileNameOut,'atf'];	%Data output filename
   fidOF=fopen(FileNameOut,'w');	%Opens trace file for writing
   for i=1:10	%Reading text file header
      DataNotes{i}=fgets(fidDF);
   end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
   
   mx=sscanf(DataNotes{5},'%6c %f %2c');
   max2=mx(7);
   mx=sscanf(DataNotes{6},'%9c %f %2c');
   min2=mx(10);
   DataNotes{2}=[sprintf('%s\t','7'), sprintf('%s\n','3')];
   DataNotes{5}=[sprintf('%s','"YTop='), sprintf('%f',max2), sprintf('%s',','), sprintf('%f',max3), sprintf('%s\n','"')];
   DataNotes{6}=[sprintf('%s','"YBottom='), sprintf('%f',min2), sprintf('%s',','), sprintf('%f',min3), sprintf('%s\n','"')];
   DataNotes{8}=sprintf('%s\n','"SignalsExported=Ipatch,Pressure"');
   DataNotes{9}=sprintf('%s\n','"Signals="	"Ipatch"	"Pressure"');
   DataNotes{10}=sprintf('%s\n','"Time (s)"	"Trace #1 (pA)"	"Trace #1 (mmHg)"');
   
   for i=1:10	%Writing text file header
      Lines=fprintf(fidOF,'%s',DataNotes{i});	%Prints  1 line with line terminators from the file
   end
   
   XY=[0;0];
   Read=2*ReadStep;
   while Read>=2*ReadStep	%If the last time complete ReadStep length was read
	   [XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
	   if Read~=0
		   Pressure=interp1AGA(TimePres(1,:),TimePres(2,:),XY(1,:),'linear')+PS(1+mod((1:Read/2),length(PS)));	%Adds pressure sample file to the calculated baseline
		   TCP=[XY(1,:);XY(2,:);Pressure];	%Time, current, pressure matrix
		   fprintf(fidOF,'%f\t %g\t %g\n',TCP);
		   SndN(1);
	   end 
   end
case 3	%Text File (.txt)
end

fclose(fidDF);	%Closes Data input file
fclose(fidOF);	%Closes Data output file

SndN(2);
