%Function for baseline drift correction
function baselncor(FileName,FileType,MinT,MaxT,TimeStep,StdRng,Chnl);

%Inputs:
%MinT -	%Start time of baseline sample
%MaxT -	%End time of baseline sample
%TimeStep (in seconds) for making baseline correction node
%StdRng - 	%Range of points to include into averaging, +- this number of Standard deviations
%Chnl	- Channel to correct baseline
Chnl=Chnl+1;	%Add time column
MinBslnPoints=0.05;	%Minimal part of points which should fit into baseline range. Otherwise no baseline points will be set in the step
if MaxT==0|MaxT<=MinT
    MaxT=MinT+TimeStep;
end

ReadStep=200000;	%Data portion to read from HDD in a step

FileNameOut=[FileName,'_bsl.'];	%Data output filename
fidDF=fopen(FileName);	%Opens trace file
switch FileType
case 1	%Axon Text File (.ATF) with 2 channels
    FileNameOut=[FileNameOut,'atf'];	%Data output filename
    fidOF=fopen(FileNameOut,'w');	%Opens trace file for writing
    fidHF=fopen([FileNameOut,'.tmp'],'w');	%Header file for writing
    
    for i=1:10	%Reading text file header
		DataNotes{i}=fgets(fidDF);
    end
	% Correct header according to acquisition mode
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
	
    mx=sscanf(DataNotes{5},'%6c %f %1c %f %2c');
    max2=mx(7); max3=mx(9);
    mx=sscanf(DataNotes{6},'%9c %f %1c %f %2c');
    min2=mx(10); min3=mx(12);
    
    
    XY=[0;0;0];
    Read=3*ReadStep;
    DelT=0;	%time shift initial value
    BslSampl=[];	%baseline sample piece
    
    
    %********* Baseline Sample reading
    while Read>=3*ReadStep&DelT==0	%If the last time complete ReadStep length was read
	[XY,Read]=fscanf(fidDF,'%g %g %g',[3 ReadStep]);
	if Read~=0	%There were som data read
	    if XY(1,end)>MinT	%Beginning of baseline sample is in the region
		if XY(1,end)>MaxT	%End of baseline sample is in the region
		    if length(BslSampl)==0	%this is the full fit
			BslSampl=XY(:,(XY(1,:)>MinT&XY(1,:)<MaxT)); %full baseline sample
			DelT=1;
		    else
			BslSampln=XY(:,(XY(1,:)<MaxT)); 	%addition
			BslSampl(:,(end:(end+size(BslSampln,2))))=BslSampln(:,1:end); %full baseline sample
			DelT=1;
		    end
		else
		    BslSampl=XY(:,XY(1,:)>MinT);	%beginning of the sample
		end
		
	    end
	    SndN(1)
	end
    end
    
    %********* Baseline Sample Statistics
    StdBsl=std(BslSampl(Chnl,:));
    MeanBsl=mean(BslSampl(Chnl,:));
    ReadStep=round(TimeStep/(BslSampl(1,2)-BslSampl(1,1)));	%New read step equal to averaging time
    clear BslSampl;
    
    %******** Reset File reading position to the data beginning
    fclose(fidDF);	%Closes Data input file
    fidDF=fopen(FileName);	%Opens trace file
    
    for i=1:10	%Reading text file header
		DataNotes{i}=fgets(fidDF);
    end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
    XY=[0;0;0];
    Read=3*ReadStep;
    
    
    %******* Baseline calculation
    
    PrevStep=0;	%Flag for existing previous step
    while Read>=3*ReadStep	%If the last time complete ReadStep length was read
	
	%====== 	Read big piece of data
	[XY,Read]=fscanf(fidDF,'%g %g %g',[3 ReadStep]);
	if Read~=0	%There were some data read
	    XYBaseN=find((XY(Chnl,:)>(MeanBsl(end)-StdBsl*StdRng))&(XY(Chnl,:)<(MeanBsl(end)+StdBsl*StdRng)));	%Coordinates of baseline points
	    if length(XYBaseN)>=size(XY,2)*MinBslnPoints	%there are enough baseline points for making correction
		
		%------	Calculate baseline point
		XYBase=XY(:,XYBaseN);	%Baseline points array
		if PrevStep==1	%There was previous step
		    MeanBsl(end+1)=mean(XYBase(Chnl,:));	%Mean baseline value
		    MeanBslCrd(end+1)=XY(1,floor(Read/6));	%Time for the current baseline node
		else	%The first step
		    MeanBsl=mean(XYBase(Chnl,:));	%Mean baseline value
		    MeanBslCrd=XY(1,floor(Read/6));	%Time for the current baseline node
		    PrevStep=1;
		end
	    end
	end
    end	%Finish baseline calculation
    
    %******** Reset File reading position to the data beginning
    fclose(fidDF);	%Closes Data input file
    fidDF=fopen(FileName);	%Opens trace file
    
    for i=1:10	%Reading text file header
		DataNotes{i}=fgets(fidDF);
    end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
    XY=[0;0;0];
    %*********** Initial MIN and MAX calculation
    %====== 	Read piece of data
    [XY,Read]=fscanf(fidDF,'%g %g %g',[3 2]);
    if Read~=0	%There were som data read
	
	%======	Correction cycle
	Baseline=interp1AGB(MeanBslCrd,MeanBsl,XY(1,:),'linear');	%New corrected values
	XY(Chnl,:)=XY(Chnl,:)-Baseline;
	%Extremums calculation
	if Chnl==2	
	    min2=min(XY(2,:));
	    max2=max(XY(2,:));
	else
	    min3=min(XY(3,:));
	    max3=max(XY(3,:));
	end
	fprintf(fidOF,'%f\t %g\t %g\n',XY);
    end
    
    Read=3*ReadStep;
    
    %*********** Baseline correction and file record
    while Read>=3*ReadStep	%If the last time complete ReadStep length was read
	%====== 	Read big piece of data
	[XY,Read]=fscanf(fidDF,'%g %g %g',[3 ReadStep]);
	if Read~=0	%There were som data read
	    
	    %======	Correction cycle
	    Baseline=interp1AGB(MeanBslCrd,MeanBsl,XY(1,:),'linear');	%New corrected values
	    % % 	Baseline(1:10)	%!
	    % % 	Baseline(end-10:end)	%!
	    XY(Chnl,:)=XY(Chnl,:)-Baseline;
	    %Extremums calculation
	    if Chnl==2	
		min2=min(min2,min(XY(2,:)));
		max2=max(max2,max(XY(2,:)));
	    else
		min3=min(min3,min(XY(3,:)));
		max3=max(max3,max(XY(3,:)));
	    end
	    fprintf(fidOF,'%f\t %g\t %g\n',XY);
	end
    end	%Finish baseline correction
    % %     XY(:,1:10)	%!
    % %     XY(:,(end-10):end)	%!
    % %     MeanBslCrd
    % %     MeanBsl
    
    %Update extremums' values
    DataNotes{5}=[sprintf('%s','"YTop='), sprintf('%f',max2), sprintf('%s',','), sprintf('%f',max3), sprintf('%s\n','"')];
    DataNotes{6}=[sprintf('%s','"YBottom='), sprintf('%f',min2), sprintf('%s',','), sprintf('%f',min3), sprintf('%s\n','"')];
    
    for i=1:10	%ReWriting text file header with the updated extremums' values
	Lines=fprintf(fidHF,'%s',DataNotes{i});	%print 1 line with line terminators to the file
    end
    fclose(fidOF);	%Closes Data output file
    fclose(fidHF);	%Closes Header output file
    CommandString=['!copy "',FileNameOut,'.tmp" + "',FileNameOut,'" tmpfile.tmp'];
    eval(CommandString);
    CommandString=['!copy tmpfile.tmp "',FileNameOut,'"'];
    eval(CommandString);
    delete('tmpfile.tmp');
    delete([FileNameOut,'.tmp']);
    
    
case 2	%Axon Text File (.ATF) with 1 channel
    
    
    FileNameOut=[FileNameOut,'atf'];	%Data output filename
    fidOF=fopen(FileNameOut,'w');	%Opens trace file for writing
    fidHF=fopen([FileNameOut,'.tmp'],'w');	%Header file for writing
    
    for i=1:10	%Reading text file header
		DataNotes{i}=fgets(fidDF);
    end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
	
    
    mx=sscanf(DataNotes{5},'%6c %f %2c');
    max2=mx(7);
    mx=sscanf(DataNotes{6},'%9c %f %2c');
    min2=mx(10);
    
    
    %    XY=[0;0];
    %    Read=2*ReadStep;
    %    DelT=0;	%time shift initial value
    %    while Read>=2*ReadStep	%If the last time complete ReadStep length was read
    % 	[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
    
    XY=[0;0];
    Read=2*ReadStep;
    DelT=0;	%time shift initial value
    BslSampl=[];	%baseline sample piece
    
    
    %********* Baseline Sample reading
    while Read>=2*ReadStep&DelT==0	%If the last time complete ReadStep length was read
	
	
	[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
	if Read~=0	%There were som data read
	    if XY(1,end)>MinT	%Beginning of baseline sample is in the region
		if XY(1,end)>MaxT	%End of baseline sample is in the region
		    if length(BslSampl)==0	%this is the full fit
			BslSampl=XY(:,(XY(1,:)>MinT&XY(1,:)<MaxT)); %full baseline sample
			DelT=1;
		    else
			BslSampln=XY(:,(XY(1,:)<MaxT)); 	%addition
			BslSampl(:,(end:(end+size(BslSampln,2))))=BslSampln(:,1:end); %full baseline sample
			DelT=1;
		    end
		else
		    BslSampl=XY(:,XY(1,:)>MinT);	%beginning of the sample
		end
		
	    end
	    SndN(1)
	end
    end
    
    %********* Baseline Sample Statistics
    StdBsl=std(BslSampl(2,:));
    MeanBsl=mean(BslSampl(2,:));
    ReadStep=round(TimeStep/(BslSampl(1,2)-BslSampl(1,1)));	%New read step equal to averaging time
    clear BslSampl;
    
    %******** Reset File reading position to the data beginning
    fclose(fidDF);	%Closes Data input file
    fidDF=fopen(FileName);	%Opens trace file
    
    for i=1:10	%Reading text file header
		DataNotes{i}=fgets(fidDF);
    end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
    XY=[0;0];
    Read=2*ReadStep;
    
    
    %******* Baseline calculation
    
    PrevStep=0;	%Flag for existing previous step
    while Read>=2*ReadStep	%If the last time complete ReadStep length was read
	
	%====== 	Read big piece of data
	[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
	if Read~=0	%There were som data read
	    XYBaseN=find((XY(2,:)>(MeanBsl(end)-StdBsl*StdRng))&(XY(2,:)<(MeanBsl(end)+StdBsl*StdRng)));	%Coordinates of baseline points
	    if length(XYBaseN)>=size(XY,2)*MinBslnPoints	%there are enough baseline points for making correction
		
		%------	Calculate baseline point
		XYBase=XY(:,XYBaseN);	%Baseline points array
		if PrevStep==1	%There was previous step
		    MeanBsl(end+1)=mean(XYBase(2,:));	%Mean baseline value
		    MeanBslCrd(end+1)=XY(1,floor(Read/6));	%Time for the current baseline node
		else	%The first step
		    MeanBsl=mean(XYBase(2,:));	%Mean baseline value
		    MeanBslCrd=XY(1,floor(Read/6));	%Time for the current baseline node
		    PrevStep=1;
		end
	    end
	end
    end	%Finish baseline calculation
    
    %******** Reset File reading position to the data beginning
    fclose(fidDF);	%Closes Data input file
    fidDF=fopen(FileName);	%Opens trace file
    
    for i=1:10	%Reading text file header
		DataNotes{i}=fgets(fidDF);
    end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
    XY=[0;0];
    
    %*********** Initial MIN and MAX calculation
    %====== 	Read piece of data
    [XY,Read]=fscanf(fidDF,'%g %g',[2 2]);
    if Read~=0	%There were som data read
	
	%======	Correction cycle
	Baseline=interp1AGB(MeanBslCrd,MeanBsl,XY(1,:),'linear');	%New corrected values
	XY(Chnl,:)=XY(Chnl,:)-Baseline;
	%Extremums calculation
	min2=min(XY(2,:));
	max2=max(XY(2,:));
	fprintf(fidOF,'%f\t %g\n',XY);
    end
    
    Read=2*ReadStep;
    
    %*********** Baseline correction and file record
    while Read>=2*ReadStep	%If the last time complete ReadStep length was read
	%====== 	Read big piece of data
	[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
	if Read~=0	%There were som data read
	    
	    %======	Correction cycle
	    Baseline=interp1AGB(MeanBslCrd,MeanBsl,XY(1,:),'linear');	%New corrected values
	    % % 	Baseline(1:10)	%!
	    % % 	Baseline(end-10:end)	%!
	    XY(Chnl,:)=XY(Chnl,:)-Baseline;
	    %Extremums calculation
	    min2=min(min2,min(XY(2,:)));
	    max2=max(max2,max(XY(2,:)));
	    fprintf(fidOF,'%f\t %g\n',XY);
	end
    end	%Finish baseline correction
    % %     XY(:,1:10)	%!
    % %     XY(:,(end-10):end)	%!
    % %     MeanBslCrd
    % %     MeanBsl
    
    %Update extremums' values
    DataNotes{5}=[sprintf('%s','"YTop='), sprintf('%f',max2), sprintf('%s\n','"')];
    DataNotes{6}=[sprintf('%s','"YBottom='), sprintf('%f',min2), sprintf('%s\n','"')];
    
    for i=1:10	%ReWriting text file header with the updated extremums' values
	Lines=fprintf(fidHF,'%s',DataNotes{i});	%print 1 line with line terminators to the file
    end
    fclose(fidOF);	%Closes Data output file
    fclose(fidHF);	%Closes Header output file
    CommandString=['!copy "',FileNameOut,'.tmp" + "',FileNameOut,'" tmpfile.tmp'];
    eval(CommandString);
    CommandString=['!copy tmpfile.tmp "',FileNameOut,'"'];
    eval(CommandString);
    pause(1)
    %!!!restore afterwards 
    delete('tmpfile.tmp');
    %!!!restore afterwards 
    delete([FileNameOut,'.tmp']);
end

fclose(fidDF);	%Closes Data input file
SndN(2);
