%findextrhst - Finding extremums of histogram, and assigning minimums and boundaries as bins
function [BinsXn,MaxsXn] =findextrhst(HstX,StepX,BinsX,ExtrFindMethod)
if nargin==3
    ExtrFindMethod='MinMax';
end
switch ExtrFindMethod   %Method of extremums finding
case 'MinMax'   %All minimums and maximums search
    StepX=fix(min(StepX,length(BinsX)/3));
case 'NumMax'   %Search for the predefined number of maximums
    StepX=fix(min(StepX,length(BinsX)/3));
    HstX=averhst(HstX,1);
    TotalMax=StepX;
    %StepX=length(BinsX)/(TotalMax*10);
    StepX=1;
case 'CrosLev'  %'log P Crossection' - makes boundary at the point of crossection
        StepX=10^(StepX);   %Convert to the real scale from log scale
        MaxXtmp=max(HstX); MinXtmp=min(HstX(HstX>0));
        if StepX>=MaxXtmp|StepX<=MinXtmp    %Treshold level is bad
            StepX=(MaxXtmp*MinXtmp)^0.5;
        end    %Treshold level is bad
end   %Method of extremums finding
%dbstop in findextrhst
%Outputs:
%BinsXn -	vector of new bins, consisting of old boundaries and found minimums
%MaxsXn -	vector of new maximums coordinats

%Inputs:
%HstX -	initial histogram vector
%StepX -	number of +-neighbors to compare
%BinsX -	old bins vector

NBinsXn=[];	%Lowest bins boundary number
NMaxsXn=[];
nx=0;	%Minimum <-1>/maximum <1> flag for the last found extremum
n=1;	%Continuous extremums counter

%Extremums search
switch ExtrFindMethod   %Method of extremums finding
case {'MinMax', 'NumMax'}   %All minimums and maximums search %Search for the predefined number of maximums
    if length(HstX)>2*StepX	%Cycle for maximums definition
        for i=1:length(HstX)
            lX=max(1,i-StepX); rX=min(length(HstX),i+StepX);
            if min(HstX(lX:rX))~=max(HstX(lX:rX))
                if HstX(i)==max(HstX(lX:rX))	%Maximum
                    switch nx	%Type of the last extremum
                    case -1	%the last extremum was minimum
                        NMaxsXn(end+1)=i;
                        n=1;
                        nx=1;
                    case 0	%the first minimum
                        NMaxsXn(1)=i;
                        n=1;
                        nx=1;
                    case 1	%the last extremum was maximum
                        NMaxsXn(end)=NMaxsXn(end)*(n-1)/n+i/n;	%Average number between minimums
                        n=n+1;
                    end
                elseif HstX(i)==min(HstX(lX:rX))	%Minimum
                    switch nx	%Type of the last extremum
                    case -1	%the last extremum was minimum
                        n=n+1;
                        NBinsXn(end)=NBinsXn(end)*(n-1)/n+i/n;	%Average number between minimums
                    case 0	%the first minimum
                        NBinsXn(1)=i;
                        n=1;
                        nx=-1;
                    case 1	%the last extremum was maximum
                        NBinsXn(end+1)=i;
                        n=1;
                        nx=-1;
                    end
                end	%Maximum
            end
        end
    end	%Cycle for maximums definition
case 'CrosLev'  %'log P Crossection' - makes boundary at the point of crossection
    NCrosXn=sign(HstX-StepX);   %Define if current bin content is larger than treshold level
    NCrosXn(NCrosXn==0)=-1; %All the exact hits considered as below
    NCrosXn=NCrosXn(2:end)-NCrosXn(1:(end-1));  %Define signs of corssins of treshold
    NCrosXnUp=find(NCrosXn==2); %Positions of up-going treshold crossections
    NCrosXnDown=find(NCrosXn==-2)+1; %Positions of down-going treshold crossections
    NCrosXn=sort([NCrosXnUp,NCrosXnDown]);  %bin boundaries at crossection positions
    NCrosXnDown=NCrosXnDown(find(NCrosXnDown>NCrosXnUp(1)));    %left only down transitions after up transitions
    for ic=1:min([length(NCrosXnUp),length(NCrosXnDown)])   %cycle through the maximums
        NMaxsXn(ic)=round((NCrosXnUp(ic)+NCrosXnDown(ic))/2);   %maximum as the middle between up and down
    end  %cycle through the maximums
    if length(NMaxsXn)==0   %no maximums detected
        NMaxsXn=1;
    end
    %NMaxsXn
end  %Method of extremums finding


%*** Creation of bins values
switch ExtrFindMethod   %Method of extremums finding
case 'MinMax'   %All minimums and maximums search
    if NBinsXn(1)<=NMaxsXn(1)	%The first is minimum
        NBinsXn(1)=1;
    else	%The first is maximum
        NBinsXn=[1,NBinsXn]; %Adding the first bin
    end
    
    if NBinsXn(end)>=NMaxsXn(end)	%The last is minimum
        NBinsXn(end)=length(BinsX);
    else	%The first is maximum
        NBinsXn=[NBinsXn,length(BinsX)]; %Adding the lest bin
    end

    BinsXn(1)=BinsX(1);
    for i=2:(length(NBinsXn)-1)
        n=NBinsXn(i)+0.5;
        BinsXn(i)=BinsX(floor(n))*(1-n+floor(n))+BinsX(ceil(n))*(n-floor(n));
    end
    BinsXn(end+1)=BinsX(end);
case 'NumMax'   %Search for the predefined number of maximums
    if 1==NMaxsXn(1)	%The first is maximum
       NMaxsXn(1)=[]; %Removing the first maximum
    end
    
    if length(BinsX)==NMaxsXn(end)	%The last is maximum
       NMaxsXn(end)=[]; %Removing the last maximum
    end
    
    NMaxsXnValues=HstX(NMaxsXn);    %values at the maximums
    [NMaxsXnValues,NMaxsXnSort]=sort(NMaxsXnValues);    %sort maximums by value in the bin
    NMaxsXn=NMaxsXn(NMaxsXnSort);   %reorder numbers in sorted order
    NMaxsXn=NMaxsXn(end:-1:(end-TotalMax+1));  %take the top selected number of maximums
    NMaxsXn=sort(NMaxsXn);  %resort them back in the order of bin number
    BinsXn(1)=BinsX(1);
    for i=2:length(NMaxsXn)
        BinsXn(i)=(BinsX(NMaxsXn(i))+BinsX(NMaxsXn(i-1)))/2;    %mean position between maximums
    end
    BinsXn(end+1)=BinsX(end);

%    TotalMax;
case 'CrosLev'  %'log P Crossection' - makes boundary at the point of crossection
    if 1==NCrosXn(1)	%The first is maximum
       NCrosXn(1)=[]; %Removing the first maximum
    end
    
    if length(BinsX)==NCrosXn(end)	%The last is maximum
       NCrosXn(end)=[]; %Removing the last maximum
    end
    
    BinsXn(1)=BinsX(1);
    for i=1:length(NCrosXn)
        BinsXn(i+1)=BinsX(NCrosXn(i));    %Bin boundary value
    end
    BinsXn(end+1)=BinsX(end);

end   %Method of extremums finding

for i=1:length(NMaxsXn)
    n=NMaxsXn(i)+0.5;
    MaxsXn(i)=BinsX(floor(n))*(1-n+floor(n))+BinsX(ceil(n))*(n-floor(n));
end

%BinsXn	%!
%MaxsXn	%!
%NBinsXn	%!
%NMaxsXn	%!%

%figure	%!
%plot(HstX)	%!
