% landint	- program to interpolate 2D landscape based on several energy points, and plot this landscape

% _________________________ User-defined parameters

Gamma=0;	% Applied tension (Gamma) (in dyne/cm)
V=0;	% Applied voltage (V) (in mV)

% __________ Create subsets of points
% ___ Inner boundaries
innerBoundary=[...
		5.6 4 50;...
		7.2 5.07 50;...
		9.2 4.12 45;...
		
		9.4 0.53 45;...
		11 1 50;...
		12.6 1.44 50;...
		13.2 1.1 47;...
		15 0.85 40;...
		
		4 -2.3 34;...
		6 -1.8 39;...
		9 -2.5 45;...
		12 -2.7 43;....
		14 -2.5 40;....
		
		18 -2 45;....
		21 -1.5 45;....
		22 2 50;....
		18 4 40;....
	];

% ___ Energy wells
closedState=[0 0 0];
% openState=[18 0.8 24];
openState=[18 0.8 20];
subState=[14 5 26];
inactState=[6 7 8];


% ___ Energy barriers
closed_open=[...
		4 -0.15 28;...
		6 -0.3 33;...
		9 -1 35;...
		11 -1.2 35;....
		13.2 -0.8 34;....
		14.8 -0.6 33;....
	];
closed_sub=[...
		10.4 2.3 35;...
		9 1.8 38;...
	];
closed_inact=[2.2 5 45];
open_sub=[15.7 3 35];
sub_inact=[10 6 39];

	
% ___ Outer boundary
meshSize=70;

dAmin=-6;	dAmax=25;
dA=dAmin:dAmax;
dAintpl=dAmin:(dAmax-dAmin)/meshSize:dAmax;

dQmin=-6;	dQmax=9;
dQ=dQmin:dQmax;
dQintpl=dQmin:(dQmax-dQmin)/meshSize:dQmax;

oE=60;	% Energy (in kT) of the outer boundary (coordinates in A,Q,E (x,y,z) space)

%_____ Join the subsets into groups
energyWells=[...
		closedState;...
		openState;...
		subState;...
		inactState;...
	];

outerBoundary=[...
		[dA;dQmin*ones(size(dA));oE*ones(size(dA))]';...
		[dA;dQmax*ones(size(dA));oE*ones(size(dA))]';...
		[dAmin*ones(size(dQ));dQ;oE*ones(size(dQ))]';...
		[dAmax*ones(size(dQ));dQ;oE*ones(size(dQ))]';...
	];

energyBarriers=[...
		closed_open;...
		closed_sub;...
		closed_inact;...
		open_sub;...
		sub_inact;...
	];


	
% __________ Tilt the landscape according to the voltage V and tension Gamma
% allPoints(:,3)=allPoints(:,3)-Gamma*allPoints(:,1)/4.047;
outerBoundary(:,3)=outerBoundary(:,3)-Gamma*outerBoundary(:,1)/4.047;
innerBoundary(:,3)=innerBoundary(:,3)-Gamma*innerBoundary(:,1)/4.047;
energyWells(:,3)=energyWells(:,3)-Gamma*energyWells(:,1)/4.047;
energyBarriers(:,3)=energyBarriers(:,3)-Gamma*energyBarriers(:,1)/4.047;

% allPoints(:,3)=allPoints(:,3)-V*allPoints(:,2)/25.262;
outerBoundary(:,3)=outerBoundary(:,3)+V*outerBoundary(:,2)/25.262;
innerBoundary(:,3)=innerBoundary(:,3)+V*innerBoundary(:,2)/25.262;
energyWells(:,3)=energyWells(:,3)+V*energyWells(:,2)/25.262;
energyBarriers(:,3)=energyBarriers(:,3)+V*energyBarriers(:,2)/25.262;



% Join the subsets into single set
allPoints=[...
		outerBoundary;...
		innerBoundary;...
		energyWells;...
		energyBarriers;...
	];


% __________ Create the landscape
% [dAmesh,dQmesh,dEmesh] = griddata_AGA(allPoints(:,1),allPoints(:,2),allPoints(:,3),dAintpl',dQintpl,'cubic');
[dAmesh,dQmesh,dEmesh] = griddata_AGA(allPoints(:,1),allPoints(:,2),allPoints(:,3),dAintpl',dQintpl,'v4');

% __________ Plot the landscape and points subsets
figure
% Plot the landscape
colormap('jet')
% mh=mesh(dAmesh,dQmesh,dEmesh);
switch 2
	case 1
		mh=surf(dAmesh,dQmesh,dEmesh);
		shading interp
		set(mh,'EdgeColor','k');
	case 2 
		mh=contour(dAmesh,dQmesh,dEmesh,32);
		shading interp
	case 3
		mh=surfl(dAmesh,dQmesh,dEmesh);
		shading interp
		set(mh,'EdgeColor','k');
end
hold on;
view([10,50])

% Plot Outer boundary
% plot3(outerBoundary(:,1),outerBoundary(:,2),outerBoundary(:,3),'.','Color',[0.5 0.5 0.5],'MarkerFaceColor',[0.5 0.5 0.5]);

% Plot Inner boundaries
% plot3(innerBoundary(:,1),innerBoundary(:,2),innerBoundary(:,3),'.k');

% Plot Energy wells
% plot3(energyWells(:,1),energyWells(:,2),energyWells(:,3),'o','Color', [0 0 0],'MarkerFaceColor',[0 0 0],'MarkerSize',10);


% Plot Energy barriers
% plot3(energyBarriers(:,1),energyBarriers(:,2),energyBarriers(:,3),'o','Color', [0 0 0],'MarkerFaceColor',[1 1 1]);

title(['Energy Landscape of the channel at ',num2str(Gamma),' dyne/cm and ',num2str(V),' mV.']);
xlabel('dA, nm^2'); ylabel('dQ, e'); zlabel('dE, kT');


hold off;
% axis equal;
% axis vis3d;


