function [HstTout,HstFit,Bins,ChnlHst,ChnlBins,MultiChnlProb,MultiChnlCrnt]=multi2singlechnl(Hst_i,Bins_i,BinsType,MinChnlCrnt,MaxChnlCrnt,...
	ScaleMode_i,ChnlNum_i,IntermPlotStep_i,VaryChnlProb_i,VaryChnlCrnt_i);
%Inputs:
%Hst_i	- Initial histogram
%Bins_i	- Bins values of the initial histogram
%BinsType	- 'Boundaries' or 'Medians'
%MinChnlCrnt	- minimal value of the single channel curent range (bottom of the baseline)
%MaxChnlCrnt	- maximal value of the single channel curent range (top of the fully open channel)
%ScaleMode_i	- 	%Mode of scaling and weighting for histograms difference calculation: 'Linear', 'Log'
%ChnlNum_i	- number of distributions used to fit the histogram
%IntermPlotStep_i	- a plot with intermediate results will be done every ... calculation step
%VaryChnlProb_i	- Option to vary (scale) <1> or not <0> the total probability to see 1, 2, 3,.. n channel(s). If <0>, this probability will be just calculated based on the probability distribution for 1 channel. If <1> - it will be additionaly multiplied on the scale coefficient
%VaryChnlCrnt_i	- Option to vary (scale) <1> or not <0> the single-channel current (more than one channel). If <0>, this probability will be calculated based on the probability distribution for 1 channel.

%Outputs:
%HstTout	- scaled initial histogram - total sum of all the bins is 1
%HstFit		- result of the fit
%Bins	- Bins values of the final histogram. Can be rebinned, comparing to the initially histogram, if it was nonuniformly sapced
%ChnlHst	- reconstructed probability density histogram for the single channel
%ChnlBins	- Bins values of the final single-channel histogram. Can be rebinned, comparing to the initially histogram, if it was nonuniformly sapced
%MultiChnlProb	- (optional) scale coefficient for the total probability to see 1, 2, 3,.. n channel(s)
%MultiChnlCrnt	- (optional) scale coefficient for the single-channel current for the 1st, 2nd, 3rd,... channel


global Hst Bins BinsMedians ScaleMode ChnlNum DistrType IntermPlotStep WeightMatrix VaryChnlProb VaryChnlCrnt...
	MultiChnlProb MultiChnlCrnt ChnlBinsMediansList ChnlBinsList ChnlBinsMedians ChnlBins ComparisonRange

Hst=Hst_i;
Bins=Bins_i;
ScaleMode=ScaleMode_i;
IntermPlotStep=IntermPlotStep_i;
ChnlNum=ChnlNum_i;
VaryChnlProb=VaryChnlProb_i;
VaryChnlCrnt=VaryChnlCrnt_i;

%IntermPlotStep=100;	%Every this frame will be made a control print of fitted curve. 0 - no printing
IntermPlotStep=-IntermPlotStep;	%inverted sign is a Flag of first plot

%Convert bins to boundaries bins
switch BinsType	%BinsType	- 'Boundaries' or 'Medians'
case 'Boundaries'
	BinsMedians=(Bins(2:end)+Bins(1:(end-1)))/2;
case 'Medians'
	BinsMedians=Bins;
	Bins=[(2*Bins(1)-Bins(2)),Bins,(2*Bins(end)-Bins(end-1))];	%guessing and adding two extra bins at the beginning and the end
	Bins=(Bins(2:end)+Bins(1:(end-1)))/2;	%Guessing boundaries for the bins' medians
end

%Make sure the histogram is uniformly binned. Rebin, if necessary
if min(Bins)~=max(Bins)	%Bins are not uniform
	Bins=min(Bins):(max(Bins)-min(Bins))/(length(Bins)-1):max(Bins);
	BinsMediansNew=(Bins(2:end)+Bins(1:(end-1)))/2;
	Hst=interp1AGB(BinsMedians,Hst,BinsMedians,'linear');	%new interpolated values - no NaNs
	BinsMedians=BinsMediansNew;
end

%Scale histogram
Hst=Hst/sum(Hst);	%Sum of all the bins is 1

HstTout=Hst;	%Initialize output histogram

%Form initial histogram for the single channel
ChnlBinsMediansList=find((BinsMedians>MinChnlCrnt)&(BinsMedians<MaxChnlCrnt));
ChnlBinsList=[ChnlBinsMediansList,(ChnlBinsMediansList(end)+1)];
ChnlBinsMedians=BinsMedians(ChnlBinsMediansList);
ChnlBins=Bins(ChnlBinsList);
ChnlHst=Hst(ChnlBinsMediansList);
ComparisonRange=find(BinsMedians>=max(Bins(1),(ChnlBinsMedians(1)*ChnlNum))&BinsMedians<=(min(Bins(end),ChnlBinsMedians(end)*ChnlNum)));	%Potential range which can cover the combination of channels

%Convert histogramm to log scale and calculate Weight Matrix, if necessary
%Now all the probabilities in the bins will vary from -Inf to 0
switch ScaleMode	%Mode of scaling and weighting for difference calculation
case 'Linear'
case 'Log'
	Hst=log10([loghst(Hst)]);
case 'LogMaxWeighted'
	Hst=log10([loghst(Hst)]);
	WeightMatrixValue=(Hst-min(Hst(:)));	%Shift the histogram to the range 0:Inf
	WeightMatrixValue(WeightMatrixValue==0)=min(WeightMatrixValue(WeightMatrixValue>0));	%replace zero element(s) with the next minimal value
	CompareRange=round(length(Hst)/(10*ChnlNum));	%Range in bins for highlighting maximums
	for i=1:length(WeightMatrixValue)
		WeightMatrixPikes(i)=WeightMatrixValue(i)/mean(WeightMatrixValue(max(1,i-CompareRange):min(length(Hst),i+CompareRange)));;	%highlights pikes
	end
	WeightMatrix=WeightMatrixPikes'.*WeightMatrixValue;
end		%Mode of scaling and weighting for difference calculation


%Form an array of scaled total probabilities to see 1, 2, 3,.. n channel(s)
MultiChnlProb=ones(ChnlNum,1);	%Can vary from 0 to Inf (if necessary)

%Form an array of scaled currents of the 1st, 2nd, 3rd,.. n channel(s)
MultiChnlCrnt=ones(ChnlNum,1);	%Can vary from 0 to Inf (if necessary)

%=============	Optimal fit by random search algorithm
FitParameters=ChnlHst';	%make and one-dimensional array. -Hst is used to make all tuning parameters to change from 0 to Inf
if VaryChnlProb==1	%vary (scale) <1> or not <0> the total probability to see 1, 2, 3,.. n channel(s).
	FitParameters=[FitParameters,MultiChnlProb'];	%make and one-dimensional array.
end
if VaryChnlCrnt==1	%vary (scale) <1> or not <0> the fully open currents of the 1st, 2nd, 3rd,.. n channel(s).
	FitParameters=[FitParameters,MultiChnlCrnt'];	%make and one-dimensional array.
end

%random search settings
Npoints=length(FitParameters);	%number of independent variables
mx_stp=(20*Npoints)^2;	%Maximal evaluation steps number
%mx_stp=3;%!!!
MaxIter=round((20*Npoints)^1.5);	%Maximal iteration steps
%dev0=2;	%parameter deviation change rate
dev0=2;	%parameter deviation change rate
dev=0.1*dev0/Npoints;	%Initial Deviation for optimization
%dev=dev0/DistrNum;	%Initial Deviation for optimization
TolX=0.0001;	%Optimization parameter exit tolerance
TolFun=0.0001;	%Optimization function exit tolerance
%dif=2;	%difference exponent between expanding and shrinking
dif=2;	%difference exponent between expanding and shrinking
dir=1;	%directed <1>, Nondirected <2>, Persistive <3> (use all possible attempts) search
%msg=100;	%messaging frequency - no messages <0>, message on screen every <#> sucessfull steps
msg=1;	%messaging frequency - no messages <0>, message on screen every <#> sucessfull steps
tries=Npoints;
expadd=1;	%mod of the parameter change: exponential <1> (for 0 to Inf parameters) or additive <2> (for -Inf to Inf parameters)

%figure;

[FitParameters,Function_counts,Iterations,fval]=random_search_combfit(FitParameters,'multi2singlechnl_eval',mx_stp,MaxIter,...
	dev0,dev,TolX,TolFun,dif,dir,msg,tries,expadd);
if IntermPlotStep~=0	%If Add control plot of the fit curve
	hold off;
end	%If Add control plot of the fit curve

%---------- Extract histogram and scaling coefficients from the fit parameters
ChnlHst=FitParameters(1:length(ChnlHst));
%Retreive an array of scaled total probabilities to see 1, 2, 3,.. n channel(s) (if necessary)
if VaryChnlProb==1	%vary (scale) <1> or not <0> the total probability to see 1, 2, 3,.. n channel(s).
	MultiChnlProb=FitParameters((length(ChnlHst)+1):(length(ChnlHst)+ChnlNum));	%Can vary from 0 to Inf
end
%Retreive an array of scaled currents of the 1st, 2nd, 3rd,.. n channel(s) (if necessary)
if VaryChnlCrnt==1	%vary (scale) <1> or not <0> the fully open currents of the 1st, 2nd, 3rd,.. n channel(s).
	MultiChnlCrnt=FitParameters((end-ChnlNum+1):end);	%Can vary from 0 to Inf
end


FitParameters%!
%==================== calculate fitted distribution
HstFit=zeros(size(Hst));	%Initialize fitted distribution
%-------------- HstFit=


disp('Amplitude fit')
Function_counts%!!!
Iterations%!!!
FitParameters
hold off;

%Convert  boundaries bins to bins medians, if necessary - to have it in the same format as an input
switch BinsType	%BinsType	- 'Boundaries' or 'Medians'
case 'Boundaries'
case 'Medians'
	Bins=BinsMedians;
	ChnlBins=ChnlBinsMedians;
end



function	Out=range2inf(A,B,In)

%Inputs:
%A - lower boundary
%B - upper boundary
%In - value in the range

%Outputs:
%Out - value from -Inf to Inf

Out=-log((B-A)./(In-A)-1);

function	Out=inf2range(A,B,In)

%Inputs:
%A - lower boundary
%B - upper boundary
%In - value from -Inf to Inf

%Outputs:
%Out - value in the range

Out=A+(B-A)*(1./(1+exp(-In)));

function	Out=range2halfinf(A,B,In)

%Inputs:
%A - lower boundary
%B - upper boundary
%In - value in the range

%Outputs:
%Out - value from 0 to Inf

Out=-log((In-B)/(A-B));

function	Out=halfinf2range(A,B,In)

%Inputs:
%A - lower boundary
%B - upper boundary
%In - value from 0 to Inf

%Outputs:
%Out - value in the range

Out=B+(A-B)*exp(-In);

