%scaledata - Function to scale rough data file
function scaledata(FileName,FileType,BinsY,StartY,StepY,Column);	%Function to scale rough data file
%Inputs:
%FileName	- Filename with extension
%FileType	- Axon text file with 2 channels <1*>, Axon text file with 1 channel <2>, text file <3>
%BinsY 	- Original mean amplitudes for 0,1,2,3... channels opened. At least, 0 and 1.
%StartY	- New zero level for Y. ) channels amplitude level will be shifted to this level (and all the initial trace will be shifted.
%StepY	- Amplitude for sinle open channel in the output
%Column	- Number of the column in the rough data file wich will be processed <1>, <2*> - current, <3>.
ReadStep=200000;	%Data portion to read from HDD in a step
BinsO=StartY:StepY:(StartY+(length(BinsY)-1)*StepY);	%Output open channels currents

FileNameOut=[FileName,'_scaled.'];	%Data output filename
fidDF=fopen(FileName);	%Opens trace file
switch FileType
case 1	%Axon Text File (.ATF) with 2 channels
   FileNameOut=[FileNameOut,'atf'];	%Data output filename
   fidOF=fopen(FileNameOut,'w');	%Opens trace file for writing
   for i=1:10	%Reading text file header
      DataNotes{i}=fgets(fidDF);
   end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
   
   %   [bx,max2,dx,max3,ex]=sscanf(DataNotes{5},'%*6c %f %*1c %f %*2c');
   mx=sscanf(DataNotes{5},'%6c %f %1c %f %2c');
   max2=mx(7); max3=mx(9);
   mx=sscanf(DataNotes{6},'%9c %f %1c %f %2c');
   min2=mx(10); min3=mx(12);
   switch Column	%Block for rescaling minimums and maximums
   case 1	%time
   case 2	%Current
      max2=spline(BinsY,BinsO,max2);
      min2=spline(BinsY,BinsO,min2);
   case 3	%Pressure
      max3=spline(BinsY,BinsO,max3);
      min3=spline(BinsY,BinsO,min3);
   end
   
   DataNotes{5}=[sprintf('%s','"YTop='), sprintf('%f',max2), sprintf('%s',','), sprintf('%f',max3), sprintf('%s\n','"')];
   DataNotes{6}=[sprintf('%s','"YBottom='), sprintf('%f',min2), sprintf('%s',','), sprintf('%f',min3), sprintf('%s\n','"')];
   
   for i=1:10	%Writing text file header
	   Lines=fprintf(fidOF,'%s',DataNotes{i});	%reads 1 line with line termonatos from the file
   end
   
   XY=[0;0;0];
   Read=3*ReadStep;
   while Read>=3*ReadStep	%If the last time complete ReadStep length was read
	   [XY,Read]=fscanf(fidDF,'%g %g %g',[3 ReadStep]);
	   if Read~=0
		   XY(Column,:)=spline(BinsY,BinsO,XY(Column,:));
		   fprintf(fidOF,'%f\t %g\t %g\n',XY);
		   SndN(1)
	   end
   end
case 2	%Axon Text File (.ATF) with 1 channel
   FileNameOut=[FileNameOut,'atf'];	%Data output filename
   fidOF=fopen(FileNameOut,'w');	%Opens trace file for writing
   for i=1:10	%Reading text file header
      DataNotes{i}=fgets(fidDF);
   end
   % Correct header according to acquisition mode
   switch sscanf(DataNotes{3},'%s')
	   case {'"AcquisitionMode=GapFree"'}
	   case {'"AcquisitionMode=EpisodicStimulation"'}
		   DataNotes{3}=sprintf('%s\n','"AcquisitionMode=Gap Free"');
		   for i=7:9
			   DataNotes{i}=DataNotes{i+1};
		   end
		   DataNotes{10}=fgets(fidDF);
   end
   
   mx=sscanf(DataNotes{5},'%6c %f %2c');
   max2=mx(7);
   mx=sscanf(DataNotes{6},'%9c %f %2c');
   min2=mx(10);
   switch Column	%Block for rescaling minimums and maximums
   case 1	%time
   case 2	%Current
      max2=spline(BinsY,BinsO,max2);
      min2=spline(BinsY,BinsO,min2);
   end
   DataNotes{5}=[sprintf('%s','"YTop='), sprintf('%f',max2), sprintf('%s\n','"')];
   DataNotes{6}=[sprintf('%s','"YBottom='), sprintf('%f',min2), sprintf('%s\n','"')];
   
   for i=1:10	%Writing text file header
	   Lines=fprintf(fidOF,'%s',DataNotes{i});	%prints 1 line with line termonatos to the file
   end
   
   XY=[0;0];
   Read=2*ReadStep;
   while Read>=2*ReadStep	%If the last time complete ReadStep length was read
	   [XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
	   if Read~=0
		   XY(Column,:)=spline(BinsY,BinsO,XY(Column,:));
		   fprintf(fidOF,'%f\t %g\n',XY);
		   SndN(1)
	   end
   end
case 3	%Text File (.txt)
end

fclose(fidDF);	%Closes Data input file
fclose(fidOF);	%Closes Data output file
SndN(2);

