%//twodimhist - calculates 2D histogram of provided input matrixes or file data
function [Hst,BinsX,BinsY,BinsZ]=swtchtmhist(Readfile,XY,AutoX,MinX,MaxX,StepX,RebTypeX,AutoY,MinY,MaxY,StepY,RebTypeY,...
	AutoZ,MinZ,MaxZ,StepZ,RebTypeZ,FileType,TimeRng,TimeRngType,TimeRngProf,...
	NormZN,InterpLevZ);

%//Outputs:
%//Hst -	Output historgam matrix.
%//BinsX -	Calculated or provided X bins' boundaries vector.
%//BinsY -	Calculated or provided Y bins' boundaries vector.
%//BinsZ -	Calculated or provided Z bins' boundaries vector.

%//Inputs:
%//Readfile -	Flag for reading X and Y data from input vectors <1>, from single file <2*>.
%//XY -	Vector of initial X and Y dlata (2xn), or filename for input data file for X and Y values, depending on "Readfile" value.
%//AutoX -	Use absolute (full data range) Minimum and Maximum to range bins <1*>, or use user-defined values <0>.
%//AutoY -	Use absolute (full data range) Minimum and Maximum to range bins <1*>, or use user-defined values <0>.
%//RebTypeX -	Defines whether to use predefined steps number <1*>, bin width <2>, or user-defined bin boundaries vector <3>.
%//RebTypeY -	Defines whether to use predefined steps number <1*>, bin width <2>, or user-defined bin boundaries vector <3>.
%//MinX -	Minimal X range value.
%//StepX -	X range steps number <1*>, X range bin width <2>, or
%//user-defined bin boundaries vector <3> (depending on "StepsBinVect" value).
%//MaxX -	Maximal X range value.
%//MinY -	Minimal Y range value.
%//StepY -	Y range steps number <1*>, X range bin width <2>, or
%//user-defined bin boundaries vector <3> (depending on "StepsBinVect" value).
%//MaxY -	Maximal Y range value.
%//FileType -	Axon Text File (.ATF) <1*> with 2 channels, c	Axon Text File (.ATF) <2> with 1 channels, Text file (.TXT) <3>, or Axon binary file (.ABF) <4>.
%//TimeRng -	Range radius +- this number of points to be included in consideration
%//TimeRngType -	times which are less than minimal: <1*> - ignore, <2> - include as minimal values
%//TimeRngProf -	Profile for including points in consideration: 'Eqvl' Equal amplitude for all points considered <1>,
%//'Ln Dn' linear decreaze from 1 in the middle up to zero at the point next after the range <2>
%//NormZN -	Type of Z-points axis normalization: 'Z Absolute' <1>,'Z/(All pnts)' <2>,'Z/(X pnts)' <3>,'Z/(Y pnts)' <4>,'Z*(X wght)' <5>,'Z*(Y wght)' <6>
%//InterpLevZ -	Degree of Z-points axis normalization at weighting
global VminW VmaxW Hst SwitchVector
%dbstop swtchtmhist 608

ReadStep=200000;	%//Step for data reading from HDD

%//***	Defining Minimum and Maximum to range bins

if Readfile==2	%//reading X and Y data from single file
	DataFile=XY;	%//Data filename
	fidDF=fopen(DataFile);	%//Opens trace file
	switch FileType
	case 1	%//Axon Text File (.ATF) with 2 channels
		for i=1:10	%//Reading text file header
			DataNotes{i}=fgetl(fidDF);
		end
	case 2	%//Axon Text File (.ATF) with 1 channel
		for i=1:10	%//Reading text file header
			DataNotes{i}=fgetl(fidDF);
		end
	case 3	%//Text File (.txt)
	end
	
	if ((AutoX==1)|(AutoY==1)|(AutoZ==1))	%//Use absolute (full data range) Minimum and Maximum to range bins
		
		switch FileType
		case 1	%//Axon Text File (.ATF) with 2 channels
			[t]=sscanf(DataNotes{5},'%*6c %f %*1c %f');
			if (AutoX==1); MaxX=t(1); end
			if (AutoY==1); MaxY=t(2); end
			[t]=sscanf(DataNotes{6},'%*9c %f %*1c %f');
			if (AutoX==1); MinX=t(1); end
			if (AutoY==1); MinY=t(2); end
			
			if AutoZ==1
				XY=[0;0]; %Empty Data matrix
				t=0;	%Flag for the first step of extremum search
				Read=3*ReadStep;
				while Read>=3*ReadStep	%If the last time complete ReadStep length was read
					[XY,Read]=fscanf(fidDF,'%g %g %g',[3 ReadStep]);
					if Read~=0
						if t==0	%the first step of extremum search
							MinZ=XY(1,2)-XY(1,1);	%the first time step - equel time steps are supposed
							t=1;
						end
						MaxZ=XY(1,end);
						SndN(1);
					end
				end
			end
			
		case 2	%Axon Text File (.ATF) with 1 channel
			if (AutoX==1);
				MaxX=sscanf(DataNotes{5},'%*6c %f');
				MinX=sscanf(DataNotes{6},'%*9c %f');
			end
			
			if (AutoY==1);
				XY=[0;0]; %Empty Data matrix
				t=0;	%Flag for the first step of extremum search
				Read=2*ReadStep;
				while Read>=2*ReadStep	%If the last time complete ReadStep length was read
					[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
					if Read~=0
						XY([1 2],:)=XY([2 1],:);	%Swapping time and current
						if t==0	%the first step of extremum search
							MinXY=min(XY,[],2);
							MaxXY=max(XY,[],2);
							t=1;
						else
							MinXY=min(MinXY,min(XY,[],2));
							MaxXY=max(MaxXY,max(XY,[],2));
						end
						SndN(1);
					end
				end
				MinY=MinXY(2); MaxY=MaxXY(2); 	%Time in swapped array
			end
			
			if AutoZ==1
				XY=[0;0]; %Empty Data matrix
				t=0;	%Flag for the first step of extremum search
				Read=2*ReadStep;
				while Read>=2*ReadStep	%If the last time complete ReadStep length was read
					[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
					if Read~=0
						if t==0	%the first step of extremum search
							MinZ=XY(1,2)-XY(1,1);	%the first time step
							t=1;
						end
						MaxZ=XY(1,end);
						SndN(1);
					end
				end
			end
			
		case 3	%Text File (.txt)
			% 			XY=[0;0]; %Empty Data matrix
			% 			t=0;	%Flag for the first step of extremum search
			% 			Read=2*ReadStep;
			% 			while Read>=2*ReadStep	%If the last time complete ReadStep length was read
			% 				[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
			% 				if Read~=0
			% 					if t==0	%the first step of extremum search
			% 						MinXY=min(XY,[],2);
			% 						MaxXY=max(XY,[],2);
			% 						t=1;
			% 					else
			% 						MinXY=min(MinXY,min(XY,[],2));
			% 						MaxXY=max(MaxXY,max(XY,[],2));
			% 					end
			% 					SndN(1);
			% 				end
			% 			end
			% 			if (AutoX==1); MinX=MinXY(1); MaxX=MaxXY(1); end
			% 			if (AutoY==1); MinY=MinXY(2); MaxY=MaxXY(2); end
		end
	end
	
	
else	%reading X and Y data from input vectors
	if ((AutoX==1)|(AutoY==1))	%Use absolute (full data range) Minimum and Maximum to range bins
		MinXY=min(XY,[],2);
		MaxXY=max(XY,[],2);
		if (AutoX==1); MinX=MinXY(1); MaxX=MaxXY(1); end
		if (AutoY==1); MinY=MinXY(2); MaxY=MaxXY(2); end
		if (AutoZ==1); MinZ=XY(3,2)-XY(3,1); MaxY=MaxXY(3); end
	end
	
end



%***	Creating bins' boundaries vectors

switch RebTypeX	%Bins creation mode parameter
case 1	%predefined steps number
	BinsX=MinX:(MaxX-MinX)/StepX:MaxX;
case 2	%bin width 
	BinsX=MinX:StepX:(MaxX+StepX);
case 3	%user-defined bin boundaries vector
	BinsX=StepX;
end
BinsXC=(BinsX(1:(end-1))+BinsX(2:end))/2;	%Vector of bin's centers

switch RebTypeY	%Bins creation mode parameter
case 1	%predefined steps number
	BinsY=MinY:(MaxY-MinY)/StepY:MaxY;
case 2	%bin width 
	BinsY=MinY:StepY:(MaxY+StepY);
case 3	%user-defined bin boundaries vector
	BinsY=StepY;
end


switch RebTypeZ	%Bins creation mode parameter    - time Bins are always converted here to logarithmic scale
case 1	%predefined steps number
	if MinZ<=0;MinZ=MaxZ/10^8;end	%Removing ocassional zero time
	MinZ=log(MinZ); MaxZ=log(MaxZ);	%Logarithming timescale
	BinsZ=MinZ:(MaxZ-MinZ)/StepZ:MaxZ;
case 2	%bin width 
	if MinZ<=0;MinZ=MaxZ/10^8;end	%Removing ocassional zero time
	MinZ=log(MinZ); MaxZ=log(MaxZ); StepZ=log(StepZ);	%Logarithming timescale and step
	BinsZ=MinZ:StepZ:(MaxZ+StepZ);
case 3	%user-defined bin boundaries vector
	BinsZ=log(StepZ);	%Logarithming timescale bins vecto
end


Hst.up=zeros((length(BinsX)-1),(length(BinsY)-1),(length(BinsZ)-1));	%Initializing switching-up times Histogram file
Hst.down=Hst.up;	%Initializing switching-down times Histogram file
SwitchVector=zeros((length(BinsX)-1),3);	%SwitchVector -	Vector along the X axis with settings "unvisited" (0),  "up" (1) or "down" (-1)
				%, exact last Y values for each point, and times of the last visits. Initially this vector iz filled by zeros

				
PrevPoint=[];	%Indicator that there were not any points analysed

%----Finding minimum and maximum changes in amplitudes (and rates)
VmaxW=(BinsX(end)-BinsX(1));
VminW=VmaxW/length(BinsX);


%******************************* Distributing data among bins

if Readfile==2	%reading X and Y data from single file
	status = fseek(fidDF,0,'bof');
	XY=[0;0]; %Empty Data matrix
	Step=1;	%Flag for the first step in file reading
	XYadd=[];	%Addition for the averaging
	
	switch FileType
	case 1	%Axon Text File (.ATF) with 2 channels
		for i=1:10	%Reading text file header
			t=fgetl(fidDF);
		end
		Read=3*ReadStep;
		while Read>=3*ReadStep	%If the last time complete ReadStep length was read
			[XY,Read]=fscanf(fidDF,'%g %g %g',[3 ReadStep]);
			XY([1 2 3],:)=XY([2 3 1],:);	%Moving time to the third column.
			
			if Read~=0%
				switch NormZN	%Pre-binning data matrix conversion
				case {5, 6}	%Weighting Z-points
					[XY,XYadd]=elongate(XY,XYadd,Step,TimeRng);
				end
				[PrevPoint]=distribute(XY,BinsX,BinsY,BinsZ,BinsXC,TimeRng,TimeRngType,TimeRngProf,...
					NormZN,InterpLevZ,PrevPoint);	%Filling histogram file
			end
		end
		
	case 2	%Axon Text File (.ATF) with 1 channel
		for i=1:10	%Reading text file header
			t=fgetl(fidDF);
		end
		Read=2*ReadStep;
		while Read>=2*ReadStep	%If the last time complete ReadStep length was read
			[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
			if Read~=0
				switch NormZN	%Pre-binning data matrix conversion
				case {5, 6}	%Weighting Z-points
					[XY,XYadd]=elongate(XY,XYadd,Step,TimeRng);
				end
				XY([1 2],:)=XY([2 1],:);	%Swapping time and current
				XY(3,:)=XY(2,:);	%Duplicating time column
				[PrevPoint]=distribute(XY,BinsX,BinsY,BinsZ,BinsXC,TimeRng,TimeRngType,TimeRngProf,...
					NormZN,InterpLevZ,PrevPoint);	%Filling histogram file
			end
			
		end
		
	case 3	%Text File (.txt)
		Read=2*ReadStep;
		while Read>=2*ReadStep	%If the last time complete ReadStep length was read
			[XY,Read]=fscanf(fidDF,'%g %g',[2 ReadStep]);
			if Read~=0
				switch NormZN	%Pre-binning data matrix conversion
				case {5, 6}	%Weighting Z-points
					[XY,XYadd]=elongate(XY,XYadd,Step,TimeRng);
				end
				[PrevPoint]=distribute(XY,BinsX,BinsY,BinsZ,BinsXC,TimeRng,TimeRngType,TimeRngProf,...
					NormZN,InterpLevZ,PrevPoint);	%Filling histogram file
			end
		end
		
	end
	
else	%reading X and Y data from input vectors
	[PrevPoint]=distribute(XY,BinsX,BinsY,BinsZ,BinsXC,TimeRng,TimeRngType,TimeRngProf,...
		NormZN,InterpLevZ,PrevPoint);	%Filling histogram file
end


%******************************* Corrects non-uniform bins width

if RebTypeX==3	%Bins vector
	s=(BinsX(end)-BinsX(1))/(length(BinsX)-1);
	for i=1:(length(BinsX)-1)
		d=(BinsX(i+1)-BinsX(i))/s;
		Hst.up(i,:,:)=Hst.up(i,:,:)/d;
		Hst.down(i,:,:)=Hst.down(i,:,:)/d;
	end
end

if RebTypeY==3	%Bins vector
	s=(BinsY(end)-BinsY(1))/(length(BinsY)-1);
	for i=1:(length(BinsY)-1)
		d=(BinsY(i+1)-BinsY(i))/s;
		Hst.up(:,i,:)=Hst.up(:,i,:)/d;
		Hst.down(:,i,:)=Hst.down(:,i,:)/d;
	end
end

if RebTypeZ==3	%Bins vector	@@@ CHeck if log-scale compensation is necessary
	s=(BinsZ(end)-BinsZ(1))/(length(BinsZ)-1);
	for i=1:(length(BinsZ)-1)
		d=(BinsZ(i+1)-BinsZ(i))/s;
		Hst.up(:,:,i)=Hst.up(:,:,i)/d;
		Hst.down(:,:,i)=Hst.down(:,:,i)/d;
	end
end

BinsZ=exp(BinsZ);

fclose(fidDF);	%Closes trace file

SndN(2);



%***************************** Functions
function [PrevPoint]=distribute(XY,BinsX,BinsY,BinsZ,BinsXC,TimeRng,TimeRngType,TimeRngProf,...
	NormZN,InterpLevZ,PrevPoint);	%Filling histogram file

%distribute - this function distributes points from XY matrix into Hst matrix
%(adds new counts to values, xeisting in Hst), according to beans vectors
global EnblSnd VminW VmaxW Hst SwitchVector
%Output:
%Hst -	Output historgam matrix.

%Inputs:
%XY - 	Data matrix.
%Hst -	Input historgam matrix.
%BinsX -	Calculated or provided X bins' boundaries vector.
%BinsY -	Calculated or provided Y bins' boundaries vector.
%BinsZ -	Calculated or provided Z bins' boundaries vector.
%BinsXC -	Centers of X (current) axis bins
%TimeRng -	Range radius +- this number of points to be included in consideration
%TimeRngType -	times which are less than minimal: <1*> - ignore, <2> - include as minimal values
%TimeRngProf -	Profile for including points in consideration: 'Eqvl' Equal amplitude for all points considered <1>,
%'Ln Dn' linear decreaze from 1 in the middle up to zero at the point next after the range <2>
%NormZN -	Type of Z-points axis normalization: 'Z Absolute' <1>,'Z/(All pnts)' <2>,'Z/(X pnts)' <3>,'Z/(Y pnts)' <4>,'Z*(X wght)' <5>,'Z*(Y wght)' <6>
%InterpLevZ -	Degree of Z-points axis normalization at weighting
%PrevPoint - 	if in is [] - it is the Indicator of the first point in analysis, otherwise it 
%has coordinates [PrevX, PrevY, PrevZ] of the trace point
%SwitchVector -	Vector along the X axis with settings "unvisited" (0),  "up" (1) or "down" (-1)
				%, exact last Y values for each point, and times of the last visits. Initially this vector iz filled by zeros
	
% SizeX=size(Hst.up,1); 
SizeY=size(Hst.up,2);	SizeZ=size(Hst.up,3);%Sizes of hystogram file in X, Y, and Z directions
% PointX=fix(SizeX/2); 
PointY=fix(SizeY/2); PointZ=fix(SizeZ/2);	%Initial positions of latest bin coordinates
PrevCellY=PointY;	%Initial position for the new Y bin search in cell switching function
PrevCellZ=PointZ;	%Initial position for the new Z bin search in cell switching function

if length(PrevPoint)==0	%the very first point in this file analysis
	i=1;
	PrevPoint=[XY(1,i),XY(2,i),XY(3,i)];
	BegCycle=2;
else
	BegCycle=1;
end


%---------Determining bins analysis range
switch NormZN	%Z-points normalization mode
case {1,2,3,4,7} 
	EndCycle=size(XY,2);	%Length of XY data matrix
case {5,6}
	%------(it is not used in switching times analysis)
	BegCycle=BegCycle+TimeRng;
	EndCycle=size(XY,2)-TimeRng;	%Length of XY data matrix
end

for i=BegCycle:EndCycle	%Cycle through XY data matrix
	
	%--------------Position finding in the switching times vector
	%** X binning
% 	if XY(1,i)<BinsX(PointX+1)	%Moving down through the bins
% 		while PointX>0	%we are above the lowest bin boundary
% 			if XY(1,i)>=BinsX(PointX)	%fit in the current bin
% 				break
% 			else
% 				PointX=PointX-1;	%1 step down
% 			end
% 		end
% 	else	%Moving up through the bin%
% 		PointX=PointX+1;	%1 step up
% 		while PointX<=SizeX	%we are below the topmost bin boundary
% 			if XY(1,i)<BinsX(PointX+1)	%fit in the current bin
% 				break
% 			else
% 				PointX=PointX+1;	%1 step up
% 			end
% 		end
% 	end
	
    CrossedBinsXC=find((BinsXC>PrevPoint(1)&BinsXC<=XY(1,i))|(BinsXC<PrevPoint(1)&BinsXC>=XY(1,i)));	%Find the X (current) bins
		%whose middles are crossected by the line between previous and current points
	
	SignDX=sign(XY(1,i)-PrevPoint(1));	%position of the new bin relative to the previous
	for nX=CrossedBinsXC
		ValX=BinsXC(nX);	%Center of the corresponding X bin
		ValY=PrevPoint(2)+(XY(2,i)-PrevPoint(2))*(ValX-PrevPoint(1))/(XY(1,i)-PrevPoint(1));	%Linear interpolation of Y (pressure or time) value in the current middle-X-bin point
		ValZ=PrevPoint(3)+(XY(3,i)-PrevPoint(3))*(ValX-PrevPoint(1))/(XY(1,i)-PrevPoint(1));	%Linear interpolation of Z (time) value in the current middle-X-bin point
		
		[PrevCellY,PrevCellZ]=SwitchCell(nX,ValX,ValY,ValZ,TimeRngType,SignDX,BinsY,...
			BinsZ,PrevCellY,PrevCellZ,SizeY,SizeZ);	%Switch the cell and add one interpolated point to the histogram
		
	end
	
% 	SignDX=sign(PointX-PrevPoint);	%position of the new bin relative to the previous
% 	switch SignDX	%position of the new bin relative to the previous
% 	case {-1,1}	%Moving down or up - new trace point is below or above X (current) bin of previous trace point
% 		if SwitchVector(PrevPoint,4)==SignDX	%Direction of movemend hasn't changed
% 			%Do nothing - this point was already switched 
% 		else	%Direction of movemend has changed - switch this point again
% 			[PrevCellY,PrevCellZ]=SwitchCell(PrevPoint,SwitchVector(PrevPoint,1),...
% 				SwitchVector(PrevPoint,2),SwitchVector(PrevPoint,3),TimeRngType,SignDX,BinsY,...
% 				BinsZ,PrevCellY,PrevCellZ,SizeY,SizeZ);	%Switch the cell and add one interpolated point to the histogram
% 			%[PrevCellY,PrevCellZ]=SwitchCell(Bin,ValX,ValY,ValZ,TimeRngType,SignDX,BinsY,BinsZ,...
% 			%PrevCellY,PrevCellZ,SizeY,SizeZ)	
% 		end
% 		for iX=(PrevPoint+SignDX):SignDX:(PointX-SignDX)	%All the bins between the previous bin and current bin, except current bin by itself
% 			ValX=BinsXC(iX);	%Center of the corresponding X bin
% 			ValY=SwitchVector(PrevPoint,2)+(XY(2,i)-SwitchVector(PrevPoint,2))*(ValX-...
% 				SwitchVector(PrevPoint,1))/(XY(1,i)-SwitchVector(PrevPoint,1));	%Linear interpolation of Y (pressure or time) value in the current middle-X-bin point
% 			ValZ=SwitchVector(PrevPoint,3)+(XY(3,i)-SwitchVector(PrevPoint,3))*(ValX-...
% 				SwitchVector(PrevPoint,1))/(XY(1,i)-SwitchVector(PrevPoint,1));	%Linear interpolation of Z (time) value in the current middle-X-bin point
% 				
% 			[PrevCellY,PrevCellZ]=SwitchCell(iX,ValX,ValY,ValZ,TimeRngType,SignDX,BinsY,...
% 				BinsZ,PrevCellY,PrevCellZ,SizeY,SizeZ);	%Switch the cell and add one interpolated point to the histogram
% 		end
% 		ValX=XY(1,i);	%Center of the corresponding X bin
% 		ValY=XY(2,i);	%Linear interpolation of Y (pressure or time) value in the current middle-X-bin point
% 		ValZ=XY(3,i);	%Linear interpolation of Z (time) value in the current middle-X-bin point
% 		
% 		[PrevCellY,PrevCellZ]=SwitchCell(iX,ValX,ValY,ValZ,TimeRngType,SignDX,BinsY,...
% 			BinsZ,PrevCellY,PrevCellZ,SizeY,SizeZ);	%Switch the cell and add one interpolated point to the histogram
% 
% 		
% 	case 0 	%New trace point is in The same X (current) bin as the previous one
% 		%------- set new cell values, but leave switching sign 
% 		if sign(XY(1,i)-SwitchVector(PrevPoint,1))==0	%Identical currents
% 			SwitchVector(PrevPoint,1)=XY(1,i);	%Set new exact X (current) value into the cell (in most cases it is the value of the middle of the bin, but at the first and last bins, and pike bins it is real current value)
% 			SwitchVector(PrevPoint,2)=XY(2,i);	%Set new exact Y (pressure or time) value into the cell (interpolated pressure value for the interpolated bins, but real pressure value for the first, last and pike bins)
% 			SwitchVector(PrevPoint,3)=XY(3,i);	%Set new exact Z (time) value for the last switching event (interpolated time value for the interpolated bins, but real time value for the first, last and pike bins)
% 		else
% 			[PrevCellY,PrevCellZ]=SwitchCell(PrevPoint,XY(1,i),...
% 				XY(2,i),XY(3,i),TimeRngType,sign(XY(1,i)-SwitchVector(PrevPoint,1)),BinsY,...
% 				BinsZ,PrevCellY,PrevCellZ,SizeY,SizeZ);	%Switch the cell and add one interpolated point to the histogram
% 		end
% 	end
	
	
	
	PointY=max(1,min(SizeY,PointY));	%Limiting the maximum value
	PrevPoint=[XY(1,i),XY(2,i),XY(3,i)];	%Taking the current point as a new Previous Point
end

SndN(1);

function [XY,XYadd]=elongate(XY,XYadd,Step,TimeRng);

%inputs:
%XY -	2-row initial data matrix
%XYadd -	New addition piece (last part of the current data matrix) for the next step
%Step -		Flag for the first step of reading file <1> - yes, <0> - no
%TimeRng -	Length of the time period (in points) to add

%Outputs:
%XY -	2-row elongated data matrix
%XYadd -	New addition piece (last part of the current data matrix) for the next step

if TimeRng~=0
	if Step==1
		Step=0;
	else
		XY=[XYadd XY];
	end
	XYadd=XY(:,(end-2*TimeRng+1):end);
end

function [Ndown,Nup]=countdownup(NormVect)
%counts maximum nunber of consecuentive steps in each direction - up and down - in current

%inputs:
%NormVect - vector to look through

%Outputs:
%Ndown		- number of consecuitive steps down
%Nup		- number of consecuitive steps up

%initial values
Ndown=1;
Nup=1;

%counting cycle
for i=2:length(NormVect)
	if NormVect(i)>NormVect(i-1)	%step up
		Ndown=1;
		Nup=Nup+1;
	elseif NormVect(i)<NormVect(i-1)
		Ndown=Ndown+1;%
		Nup=1;
	end
end


function [PrevCellY,PrevCellZ]=SwitchCell(Bin,ValX,ValY,ValZ,TimeRngType,SignDX,BinsY,BinsZ,...
	PrevCellY,PrevCellZ,SizeY,SizeZ);
%Switches the cell into the switch vector

%Inputs:
%SwitchVector - Vector along the X axis with 
				%settings "unvisited" (0),  "up" (1) or "down" (-1), exact last Y values for each point, times of the last visits.
				%. Initially this vector iz filled by zeros
%Bin -	Position of the bin to switch in the SwitchVector
%ValX - New exact X value (current)
%ValY - New exact Y value (pressure or time)
%ValZ - New exact Z value (time)
%TimeRngType -	times which are less than minimal: <1*> - ignore, <2> - include as minimal values
%SignDX - 	Sign of movement up <1>, down <-1> or staying at place <0>

global Hst SwitchVector
%@@@-----------------fix decision about the current out of range 
% CellValY=mean(SwitchVector(Bin,2),ValY);	%Mean value of Y (pressure or time) for the current switching period
% %** Y binning
% if XY(2,i)<BinsY(PrevCellY)	%Moving down through the bins
% 	while PrevCellY>0	%we are above the lowest bin boundary
% 		if CellValY>=BinsY(PrevCellY)	%fit in the current bin
% 			break
% 		else
% 			PrevCellY=PrevCellY-1;	%1 step down
% 		end
% 	end
% else	%Moving up through the bins
% 	PrevCellY=PrevCellY+1;	%1 step up
% 	while PrevCellY<=SizeY	%we are below the topmost bin boundary
% 		if CellValY<BinsY(PrevCellY+1)	%fit in the current bin
% 			break
% 		else
% 			PrevCellY=PrevCellY+1;	%1 step up
% 		end
% 	end
% end

CellValY=(SwitchVector(Bin,2)+ValY)/2;	%Mean value of Y (pressure or time) for the current switching period
%** Y binning
if CellValY<BinsY(PrevCellY)	%Moving down through the bins
	while PrevCellY>0	%we are above the lowest bin boundary
		if CellValY>=BinsY(PrevCellY)	%fit in the current bin
			break
		else
			PrevCellY=PrevCellY-1;	%1 step down
		end
	end
else	%Moving up through the bins
	%PrevCellY=PrevCellY+1;	%1 step up
	while PrevCellY<=SizeY	%we are below the topmost bin boundary
		if CellValY<BinsY(PrevCellY+1)	%fit in the current bin
			break
		else
			PrevCellY=PrevCellY+1;	%1 step up
		end
	end
end

CellValZ=ValZ-SwitchVector(Bin,3);	%Time period during which channel was closed/open
%** Z binning
if CellValZ<=0	%Zero or negative time change
	PrevCellZ=0;
else
	CellValZ=log(CellValZ);	%logarithm of time period
	if CellValZ<BinsZ(PrevCellZ)	%Moving down through the bins
		while PrevCellZ>0	%we are above the lowest bin boundary
			if CellValZ>=BinsZ(PrevCellZ)	%fit in the current bin
				break
			else
				PrevCellZ=PrevCellZ-1;	%1 step down
			end
		end
	else	%Moving up through the bins
		%PrevCellZ=PrevCellZ+1;	%1 step up
		while PrevCellZ<=SizeZ	%we are below the topmost bin boundary
			if CellValZ<BinsZ(PrevCellZ+1)	%fit in the current bin
				break
			else
				PrevCellZ=PrevCellZ+1;	%1 step up
			end
		end
	end
end

if PrevCellZ==0&TimeRngType==2	%too short or zero time period, and it is set to be counted as one minimal time period
	PrevCellZ=1;	%minimal calculated time period
end

%** Adding 1 count to histogram bin
if (PrevCellZ>0)&(PrevCellY>0)&(PrevCellZ<=SizeZ)&(PrevCellY<=SizeY)	%Point coordinates are within hisogram matrix range
	%----------- Weighting value
	Weight=1;	%Initial weight
	%weighting is not used in switching times calculation. If it should, it is necessary to solve question with TimeRngType variable - now it is used for switchtime decision whether to include subminimal times
	if SwitchVector(Bin,1)==1	%Moving up is reversed now by moving down
		Hst.up(Bin,PrevCellY,PrevCellZ)=Hst.up(Bin,PrevCellY,PrevCellZ)+Weight;
	elseif SwitchVector(Bin,1)==-1	%Moving down is reversed now by moving up
		Hst.down(Bin,PrevCellY,PrevCellZ)=Hst.down(Bin,PrevCellY,PrevCellZ)+Weight;
	end	
end

%------------------------ set new cell values

SwitchVector(Bin,2)=ValY;	%Set new exact Y (pressure or time) value into the cell (interpolated pressure value for the interpolated bins, but real pressure value for the first, last and pike bins)
SwitchVector(Bin,3)=ValZ;	%Set new exact Z (time) value for the last switching event (interpolated time value for the interpolated bins, but real time value for the first, last and pike bins)
SwitchVector(Bin,1)=SignDX; 	%Flag for the last switching event: <-1> for moving down, <1> for moving up

PrevCellY=max(1,min(SizeY,PrevCellY));	%Limiting the maximum value
PrevCellZ=max(1,min(SizeZ,PrevCellZ));	%Limiting the maximum value
