function [TempZipFileName]=unzip_AGA(ZipFileName,TempZipFileName,UnzipStage);
%Function to temporary extract the content of the zip file. Works in 2 stages.
%1 - TempUnzip: checks if this file has a 'zip' extension; if not - just makes output name the same as an input name.
%if yes - extract the content of the zip file provided, rename zip
%file to autogenerated temporary name, and then rename extracted zip file
%to the name of zip file (with zip extension remaining)
%2 - TempRemove: remove extracted file and rename initial zip file from
%temporary name back to the original name

%Inputs:

%ZipFileName - Name of the initial zip file (obligatory
%TempZipFileName - autogenerated temporary name of the zip file. Used
%only for the second removal stage. For the first stage can be set to ''
%UnzipStage - Temporal unzipping stage - 'TempUnzip','TempRemove'

switch nargin  %Autoselect stage of temporal unzipping
    case 1  %TempUnzip
        UnzipStage='TempUnzip';
    case 2  %TempRemove
        UnzipStage='TempRemove';
    case 3  %Look at the third input - it is explicit
end

switch UnzipStage %Temporal unzipping stage - 'TempUnzip','TempRemove'
    case 'TempUnzip'    %Temporal unzipping
        ZipPosition=max(findstr(upper(ZipFileName),'.ZIP'));
        if ZipPosition == (length(upper(ZipFileName))-3)   %this is a zip file
            clc=clock; TempZipFolder=['tempzip_',sprintf('%02.0f',clc(2)),'.',sprintf('%02.0f',clc(3)),'.',sprintf('%04.0f',clc(1)),'_',sprintf('%02.0f',clc(4)),'h',sprintf('%02.0f',clc(5)),'m',sprintf('%05.2f',clc(6)),'s'];
            mkdir(TempZipFolder);
            TempZipFileName=[TempZipFolder,'.zip'];
            movefile(ZipFileName,TempZipFileName);
            unzip(TempZipFileName,TempZipFolder);
            UnpackedFiles=dir(TempZipFolder);
            FirstUnpackedFile=UnpackedFiles(min(find([UnpackedFiles(:).isdir]==0))).name;
            movefile([TempZipFolder,'\',FirstUnpackedFile],ZipFileName);
            rmdir(TempZipFolder);
        else    %this is not a zip file
            TempZipFileName=ZipFileName;
        end   %this is a zip file
    case 'TempRemove'   %Removal of temporal unzipped files
        if strcmp(TempZipFileName,ZipFileName)   %this was a zip file
        else    %this was not a zip file
            movefile(TempZipFileName,ZipFileName);
        end   %this was a zip file
end %Temporal unzipping stage - 'TempUnzip','TempRemove'
